  /******************************************************************************

  Commande Kamoer KVP04
  Copyright Gilles Muller - Decembre 2020, <gm.cnc@free.fr>

  Commercial usage forbidden without permission of the author
******************************************************************************/

#include <LiquidCrystal.h>
#include <EEPROM.h>

// initialize the library with the numbers of the interface pins
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

// constantes du PID
const double Kp = 0.1;

int Consigne = 100;
double Diff;

// Automate
const int Idle = 0;
const int Demarrage = 1;
const int Running = 2;

const int PinMoteur = 11;
const int Cmin = 30;            // Valeur min du moteur pour ne pas bloquer
const int Cmin2 = 40;           // Valeur min du moteur apres la mise en pression
const int Cmax = 255;           // Valeur max du moteur

double Commande = 0;            // valeur de la commande moteur
bool up = true;
int state = Idle;
int Calib = 0;                  // Calibration pression
int PressC;                     // Pression calibree
int i = 0;                      // Frequence du PID
int eeAddress = 0;


void setup()
{
  Serial.begin(9600); // Initialize Serial to log output

  //TCCR2B = TCCR2B & B11111000 | B00000001;    // set timer 2 divisor to     1 for PWM frequency of 31372.55 Hz
  TCCR2B = TCCR2B & B11111000 | B00000010;      // set timer 2 divisor to     8 for PWM frequency of  3921.16 Hz
  pinMode(PinMoteur, OUTPUT);
  analogWrite (PinMoteur, 255);                 // Moteur off

  // set up the LCD's number of columns and rows:
  lcd.begin(16, 2);
  lcd.setCursor(0, 0);
  lcd.print("Pump V2 Keypad2");

  // Read Values from EEPROM
  eeAddress = 0;
  EEPROM.get(eeAddress, Consigne);
  eeAddress += sizeof(int);
  EEPROM.get(eeAddress, Calib);

  // Test de validité des valeurs
  if ((Consigne > 500) || (Consigne < 50))
    Consigne = 100;

  if ((Calib > 100) || (Calib < 0))
    Calib = 0;
}

void loop()
{
  double val = analogRead(0);
  int Press = analogRead(1) * 1.111; 

  lcd.setCursor(10, 0);
  lcd.print(String(Consigne) + "Kpa");

  // touche Calibration (Select)
  if (val > 800 && val < 850)
  {
    Calib = Press;          // Calibration de la pression
    eeAddress = 0;
    eeAddress += sizeof(int);
    EEPROM.put(eeAddress, Calib);
  }

  // touche démarrage (Left)
  if (val > 600 && val < 650)
  {
    eeAddress = 0;           // Sauvegarde de la consigne
    EEPROM.put(eeAddress, Consigne);
    Commande = Cmax;         // Moteur a fond
    state = Demarrage;
  }

  // touche arret (Right)
  if (val < 50)
  {
    Commande = 0;         // Moteur a l'arret
    state = Idle;
  }

  // touche up
  if (val > 190 && val < 250)
  {
    if (Consigne < 500)
      Consigne += 5;
  }
  // touche down
  else if (val > 390 && val < 450)
  {
    if (Consigne > 20)
      Consigne -= 5;
  }

  // Le PID tourne a 1Hz, alors que les touches sont a 5Hz
  i++;
  if (i == 5)
  {
    i = 0;

    PressC = Press - Calib;
    Diff = Consigne - PressC;

    lcd.setCursor(0, 1);
    lcd.print("p" + String(PressC) + "Kpa c" + String(int(Commande)) + " o" + String(Calib) + "  ");

    switch (state) {
      case Idle:
        break;

      case Demarrage:
        if (PressC > Consigne)      // On fait le vide jusqu'a la consigne
        {
          Commande = Cmin2;          // On met le moteur au min sans arreter la pompe, et on lance l'adaptation
          state = Running;
        }
        break;

      case Running:
        Commande += Diff * Kp;
        if (Commande > Cmax)
          Commande = Cmax;
        if (Commande < Cmin)        // Valeur Mini pour ne pas arreter la pompe
          Commande = Cmin;
        break;
    }

    // controle du PWM
    analogWrite (PinMoteur, 255 - ((int)Commande));
  }
  delay(200);

}
